@@ -54,6 +54,7 @@ _PROP_COLOR = "color"
 _PROP_OWNER = "owner"
 _PROP_VALID = "valid"
 _PROP_OBJID = 'objid'
+_PROP_TAGS = 'tags'
 
 # Will go away soon
 _PROP_IP4_ADDRESS = "ip4-address"
@@ -165,7 +166,10 @@ class Buddy(ExportedGObject):
         _PROP_OBJID        : (str, None, None, None, gobject.PARAM_READABLE),
         _PROP_IP4_ADDRESS  : (str, None, None, None,
                               gobject.PARAM_CONSTRUCT_ONLY |
-                              gobject.PARAM_READWRITE)
+                              gobject.PARAM_READWRITE),
+        _PROP_TAGS         : (str, None, None, None,
+                              gobject.PARAM_CONSTRUCT_ONLY |
+                              gobject.PARAM_READWRITE),
     }
 
     def __init__(self, bus, object_id, **kwargs):
@@ -198,9 +202,11 @@ class Buddy(ExportedGObject):
         self._nick = None
         self._color = None
         self._ip4_address = None
+        self._tags = None
 
         _ALLOWED_INIT_PROPS = [_PROP_NICK, _PROP_KEY, _PROP_ICON,
-                               _PROP_CURACT, _PROP_COLOR, _PROP_IP4_ADDRESS]
+                               _PROP_CURACT, _PROP_COLOR, _PROP_IP4_ADDRESS,
+                               _PROP_TAGS]
         for (key, value) in kwargs.items():
             if key not in _ALLOWED_INIT_PROPS:
                 _logger.debug("Invalid init property '%s'; ignoring..." % key)
@@ -250,6 +256,8 @@ class Buddy(ExportedGObject):
             return self._owner
         elif pspec.name == _PROP_IP4_ADDRESS:
             return self._ip4_address
+        elif pspec.name == _PROP_TAGS:
+            return self._tags
 
     def do_set_property(self, pspec, value):
         """Set given property
@@ -277,6 +285,8 @@ class Buddy(ExportedGObject):
             self._key = value
         elif pspec.name == _PROP_IP4_ADDRESS:
             self._ip4_address = value
+        elif pspec.name == _PROP_TAGS:
+            self._tags = value
 
     # dbus signals
     @dbus.service.signal(_BUDDY_INTERFACE,
@@ -410,6 +420,7 @@ class Buddy(ExportedGObject):
         props[_PROP_COLOR] = self.props.color or ''
         props[_PROP_IP4_ADDRESS] = self.props.ip4_address or ''
         props[_PROP_CURACT] = self.props.current_activity or ''
+        props[_PROP_TAGS] = self.props.tags or ''
         return props
 
     def get_identifier_by_plugin(self, plugin):
@@ -538,6 +549,12 @@ class Buddy(ExportedGObject):
                     self._key = key
                     changed_props[_PROP_KEY] = key or ''
                     changed = True
+        if _PROP_TAGS in properties:
+            tags = properties[_PROP_TAGS]
+            if tags != self._tags:
+                self._tags = tags
+                changed_props[_PROP_TAGS] = tags or ''
+                changed = True
 
         if not changed or not changed_props:
             return
@@ -696,18 +713,7 @@ class GenericOwner(Buddy):
 
         uses SetActivities on BuddyInfo channel
         """
-        conn = tp.get_connection()
-
-        if CONN_INTERFACE_BUDDY_INFO not in conn:
-            _logger.warning('%s does not support BuddyInfo - unable to '
-                            'set activities' % conn.object_path)
-            return
-
-        conn[CONN_INTERFACE_BUDDY_INFO].SetActivities(
-                self._activities_by_connection[tp].iteritems(),
-                reply_handler=_noop,
-                error_handler=lambda e:
-                    _logger.warning("setting activities failed: %s", e))
+        pass
 
     def _set_self_current_activity(self, tp):
         """Forward our current activity (or "") to network
@@ -725,35 +731,9 @@ class GenericOwner(Buddy):
                 cur_activity = ""
                 cur_activity_handle = 0
 
-        _logger.debug("Setting current activity to '%s' (handle %s)",
-                      cur_activity, cur_activity_handle)
-        conn = tp.get_connection()
-
-        if CONN_INTERFACE_BUDDY_INFO not in conn:
-            _logger.warning('%s does not support BuddyInfo - unable to '
-                            'set current activity' % conn.object_path)
-            return
-
-        self.PropertyChanged({_PROP_CURACT: self._current_activity or ''})
-        conn[CONN_INTERFACE_BUDDY_INFO].SetCurrentActivity(cur_activity,
-                cur_activity_handle,
-                reply_handler=_noop,
-                error_handler=lambda e:
-                    _logger.warning("setting current activity failed: %s", e))
-
     def _set_self_alias(self, tp):
-        self_handle = self._handles[tp][0]
-        conn = tp.get_connection()
+        pass
 
-        if CONN_INTERFACE_ALIASING not in conn:
-            _logger.warning('%s does not support aliasing - unable to '
-                            'set my own alias' % conn.object_path)
-            return False
-
-        conn[CONN_INTERFACE_ALIASING].SetAliases({self_handle: self._nick},
-                reply_handler=_noop,
-                error_handler=lambda e:
-                    _logger.warning('Error setting alias: %s', e))
         # Hack so we can use this as a timeout handler
         return False
 
@@ -768,42 +748,13 @@ class GenericOwner(Buddy):
             'color': self._color or '',
             'key': dbus.ByteArray(self._key or ''),
             'ip4-address': self._ip4_address or '',
+            'tags': self._tags or '',
             }, signature='sv')
 
         # FIXME: clarify whether we're meant to support random extra properties
         # (Salut doesn't)
-        if tp._PROTOCOL == 'salut':
-            del props['ip4-address']
-
-        if connected:
-            if CONN_INTERFACE_BUDDY_INFO not in conn:
-                _logger.warning('%s does not support BuddyInfo - unable to '
-                                'set my own buddy properties' %
-                                conn.object_path)
-                return False
-
-            conn[CONN_INTERFACE_BUDDY_INFO].SetProperties(props,
-                    reply_handler=_noop,
-                    error_handler=lambda e:
-                        _logger.warning('Error setting OLPC properties: %s', e))
-        else:
-            # we don't yet know whether the connection supports setting buddy
-            # properties
-            # FIXME: remove this hack, and the import of dbus.proxies, when
-            # we have a newer tp-python that makes dbus_object public
-            try:
-                obj = conn.dbus_object
-                if not isinstance(obj, dbus.proxies.ProxyObject):
-                    raise AttributeError
-            except AttributeError:
-                obj = conn._dbus_object
-
-            obj.SetProperties(props, dbus_interface=CONN_INTERFACE_BUDDY_INFO,
-                    reply_handler=lambda:
-                        _logger.debug('Successfully preloaded buddy props'),
-                    error_handler=lambda e:
-                        _logger.debug('Failed to preload buddy properties, '
-                                      'will try again after Connect(): %s', e))
+        if tp._PROTOCOL == 'local-xmpp':
+            del props['tags']
 
         # Hack so we can use this as a timeout handler
         return False
@@ -834,58 +785,6 @@ class GenericOwner(Buddy):
         # issues in the avatar cache. Revisit this afterwards
         return
 
-        conn = tp.get_connection()
-        icon_data = self._icon
-
-        if CONN_INTERFACE_AVATARS not in conn:
-            _logger.warning('%s does not support Avatars - unable to '
-                            'set my own avatar on this connection' %
-                            conn.object_path)
-            return
-
-        m = new_md5()
-        m.update(icon_data)
-        digest = m.hexdigest()
-
-        self_handle = self._handles[tp][0]
-        token = conn[CONN_INTERFACE_AVATARS].GetAvatarTokens(
-                [self_handle])[0]
-
-        if buddy_icon_cache.check_avatar(conn.object_path, digest,
-                                         token):
-            # avatar is up to date
-            return
-
-        def set_self_avatar_cb(token):
-            buddy_icon_cache.set_avatar(conn.object_path, digest, token)
-
-        types, minw, minh, maxw, maxh, maxsize = \
-                conn[CONN_INTERFACE_AVATARS].GetAvatarRequirements()
-        if not "image/jpeg" in types:
-            _logger.debug("server does not accept JPEG format avatars.")
-            return
-
-        width = 96
-        height = 96
-        size = 8192
-        if maxw > 0 and width > maxw:
-            width = maxw
-        if maxw > 0 and height > maxh:
-            height = maxh
-        if maxsize > 0 and size > maxsize:
-            size = maxsize
-
-        if 1:
-            # FIXME: Avatars have been disabled for Trial-2 due to performance
-            # issues in the avatar cache. Revisit this afterwards
-            pass
-        else:
-            img_data = _get_buddy_icon_at_size(icon_data, width, height, size)
-            conn[CONN_INTERFACE_AVATARS].SetAvatar(img_data, "image/jpeg",
-                    reply_handler=set_self_avatar_cb,
-                    error_handler=lambda e:
-                        _logger.warning('Error setting avatar: %s', e))
-
     def _property_changed(self, changed_props):
         for tp in self._handles.iterkeys():
 
@@ -898,7 +797,8 @@ class GenericOwner(Buddy):
                 gobject.timeout_add(1000, lambda: self._set_self_alias(tp))
 
             if (changed_props.has_key("color") or
-                changed_props.has_key("ip4-address")):
+                changed_props.has_key("ip4-address") or
+                changed_props.has_key("tags")):
                 if tp.status == CONNECTION_STATUS_CONNECTED:
                     self._set_self_olpc_properties(tp)
 
@@ -956,10 +856,10 @@ class ShellOwner(GenericOwner):
         key = profile.pubkey
 
         color = client.get_string("/desktop/sugar/user/color")
+        tags = client.get_string("/desktop/sugar/user/tags")
         nick = client.get_string("/desktop/sugar/user/nick")
+
         if not isinstance(nick, unicode):
-            # GConf let you only store utf-8 encoded strings, why we can
-            # expect an utf-8 string here
             nick = unicode(nick, 'utf-8')
 
         icon_file = os.path.join(env.get_profile_path(), "buddy-icon.jpg")
@@ -969,7 +869,8 @@ class ShellOwner(GenericOwner):
 
         GenericOwner.__init__(self, ps, bus,
                 'keyid/' + psutils.pubkey_to_keyid(key),
-                key=key, nick=nick, color=color, icon=icon, key_hash=key_hash)
+                key=key, nick=nick, color=color, icon=icon, key_hash=key_hash,
+                tags=tags)
 
         # Ask to get notifications on Owner object property changes in the
         # shell. If it's not currently running, no problem - we'll get the
@@ -977,6 +878,7 @@ class ShellOwner(GenericOwner):
         for (signal, cb) in (('IconChanged', self._icon_changed_cb),
                              ('ColorChanged', self._color_changed_cb),
                              ('NickChanged', self._nick_changed_cb),
+                             ('TagsChanged', self._tags_changed_cb),
                              ('CurrentActivityChanged',
                               self._cur_activity_changed_cb)):
             self._bus.add_signal_receiver(cb, signal_name=signal,
@@ -999,6 +901,11 @@ class ShellOwner(GenericOwner):
         props = {_PROP_COLOR: color}
         self.set_properties(props)
 
+    def _tags_changed_cb(self, tags):
+        """Handle tags change, set property to generate event"""
+        props = {_PROP_TAGS: tags}
+        self.set_properties(props)
+
     def _nick_changed_cb(self, nick):
         """Handle nickname change, set property to generate event"""
         props = {_PROP_NICK: nick}
